require 'morpheus/api/api_client'

class Morpheus::MonitoringIncidentsInterface < Morpheus::APIClient

  def get(id, params={})
    raise "#{self.class}.get() passed a blank id!" if id.to_s == ''
    url = "#{@base_url}/api/monitoring/incidents/#{id}"
    headers = { params: params, authorization: "Bearer #{@access_token}" }
    opts = {method: :get, url: url, headers: headers}
    execute(opts)
  end

  def list(params={})
    url = "#{@base_url}/api/monitoring/incidents"
    headers = { params: params, authorization: "Bearer #{@access_token}" }
    opts = {method: :get, url: url, headers: headers}
    execute(opts)
  end

  def stats(params={})
    url = "#{@base_url}/api/monitoring/incidents/stats"
    headers = { params: params, authorization: "Bearer #{@access_token}" }
    opts = {method: :get, url: url, headers: headers}
    execute(opts)
  end

  def create(options)
    url = "#{@base_url}/api/monitoring/incidents"
    headers = { :authorization => "Bearer #{@access_token}", 'Content-Type' => 'application/json' }
    payload = options
    opts = {method: :post, url: url, headers: headers, payload: payload.to_json}
    execute(opts)
  end

  def update(id, options)
    url = "#{@base_url}/api/monitoring/incidents/#{id}"
    headers = { :authorization => "Bearer #{@access_token}", 'Content-Type' => 'application/json' }
    payload = options
    opts = {method: :put, url: url, headers: headers, payload: payload.to_json}
    execute(opts)
  end

  def destroy(id, payload={})
    url = "#{@base_url}/api/monitoring/incidents/#{id}"
    headers = { :authorization => "Bearer #{@access_token}", 'Content-Type' => 'application/json' }
    opts = {method: :delete, url: url, headers: headers, payload: payload.to_json}
    execute(opts)
  end

  def close(id)
    destroy(id)
  end

  def reopen(id, options={})
    url = "#{@base_url}/api/monitoring/incidents/#{id}/reopen"
    headers = { :authorization => "Bearer #{@access_token}", 'Content-Type' => 'application/json' }
    opts = {method: :put, url: url, headers: headers, payload: options.to_json}
    execute(opts)
  end

  def mute(id, options={})
    url = "#{@base_url}/api/monitoring/incidents/#{id}/mute"
    headers = { :authorization => "Bearer #{@access_token}", 'Content-Type' => 'application/json' }
    payload = options
    opts = {method: :put, url: url, headers: headers, payload: payload.to_json}
    execute(opts)
  end

  def mute_all(payload={})
    url = "#{@base_url}/api/monitoring/incidents/mute-all"
    headers = { :authorization => "Bearer #{@access_token}", 'Content-Type' => 'application/json' }
    opts = {method: :put, url: url, headers: headers, payload: payload.to_json}
    execute(opts)
  end

  def quarantine(id, payload={})
    mute(id, payload)
  end

  def quarantine_all(payload={})
    mute_all(payload)
  end

  def history(id, params={})
    url = "#{@base_url}/api/monitoring/incidents/#{id}/history"
    headers = { params: params, authorization: "Bearer #{@access_token}" }
    opts = {method: :get, url: url, headers: headers}
    execute(opts)
  end

  def notifications(id, params={})
    url = "#{@base_url}/api/monitoring/incidents/#{id}/notifications"
    headers = { params: params, authorization: "Bearer #{@access_token}" }
    opts = {method: :get, url: url, headers: headers}
    execute(opts)
  end

  def events(id, params={})
    # JD: maybe switch to this instead /api/monitoring/incidents/#{id}/events instead?
    # url = "#{@base_url}/api/monitoring/incidents/#{id}/events"
    url = "#{@base_url}/api/monitoring/incident-events/#{id}"
    headers = { params: params, authorization: "Bearer #{@access_token}" }
    opts = {method: :get, url: url, headers: headers}
    execute(opts)
  end

end
