require 'morpheus/api/api_client'

class Morpheus::NetworkDomainRecordsInterface < Morpheus::APIClient

  def get(domain_id, id, params={})
    raise "#{self.class}.get() passed a blank id!" if id.to_s == ''
    url = "#{@base_url}/api/networks/domains/#{domain_id}/records/#{id}"
    headers = { params: params, authorization: "Bearer #{@access_token}" }
    opts = {method: :get, url: url, headers: headers}
    execute(opts)
  end

  def list(domain_id, params={})
    url = "#{@base_url}/api/networks/domains/#{domain_id}/records"
    headers = { params: params, authorization: "Bearer #{@access_token}" }
    opts = {method: :get, url: url, headers: headers}
    execute(opts)
  end

  def create(domain_id, payload)
    url = "#{@base_url}/api/networks/domains/#{domain_id}/records"
    headers = { :authorization => "Bearer #{@access_token}", 'Content-Type' => 'application/json' }
    opts = {method: :post, url: url, headers: headers, payload: payload.to_json}
    execute(opts)
  end

  def update(domain_id, id, payload)
    url = "#{@base_url}/api/networks/domains/#{domain_id}/records/#{id}"
    headers = { :authorization => "Bearer #{@access_token}", 'Content-Type' => 'application/json' }
    opts = {method: :put, url: url, headers: headers, payload: payload.to_json}
    execute(opts)
  end

  def destroy(domain_id, id, params={})
    url = "#{@base_url}/api/networks/domains/#{domain_id}/records/#{id}"
    headers = { :params => params, :authorization => "Bearer #{@access_token}", 'Content-Type' => 'application/json' }
    opts = {method: :delete, url: url, headers: headers}
    execute(opts)
  end

end
